#include <ntifs.h>
#include <string.h>
#include <fltKernel.h>  
#include "Protect.h"
//#include "Header.h"

PFLT_FILTER FilterHandle;

FLT_OPERATION_REGISTRATION Callbacks[] = 
{
	{
		IRP_MJ_ACQUIRE_FOR_SECTION_SYNCHRONIZATION ,
		0,
		PreFileSystemControl,
		NULL
	},

	{ IRP_MJ_OPERATION_END }
};

FLT_REGISTRATION FilterRegistration = 
{
    sizeof(FLT_REGISTRATION),
    FLT_REGISTRATION_VERSION,							
    0,

    NULL,
    Callbacks,

    FltUnload,

    InstanceSetup,
  
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL

};

NTSTATUS InstanceSetup (
    __in PCFLT_RELATED_OBJECTS FltObjects,
    __in FLT_INSTANCE_SETUP_FLAGS Flags,
    __in DEVICE_TYPE VolumeDeviceType,
    __in FLT_FILESYSTEM_TYPE VolumeFilesystemType
    )

{

    return STATUS_SUCCESS;
}

NTSTATUS FltUnload ( FLT_FILTER_UNLOAD_FLAGS Flags )
{

    FltUnregisterFilter( FilterHandle );

    return STATUS_SUCCESS;
}

FLT_PREOP_CALLBACK_STATUS  PreFileSystemControl( PFLT_CALLBACK_DATA Data, PCFLT_RELATED_OBJECTS FltObjects, PVOID *CompletionContext)

{
    NTSTATUS status;
	PFLT_FILE_NAME_INFORMATION nameInfo = NULL;	
	UNICODE_STRING   str;

	{
			status = FltGetFileNameInformation( Data,
												FLT_FILE_NAME_NORMALIZED |
												FLT_FILE_NAME_QUERY_ALWAYS_ALLOW_CACHE_LOOKUP,
												&nameInfo );    

 
			if (!NT_SUCCESS(status) )
			{
				status = FltGetFileNameInformationUnsafe( FltObjects->FileObject,
														  FltObjects->Instance, 
														  FLT_FILE_NAME_NORMALIZED |
														  FLT_FILE_NAME_QUERY_ALWAYS_ALLOW_CACHE_LOOKUP,
														  &nameInfo );  
			}
    
   
			if (nameInfo )
			{
				WCHAR  *lowercaseString, *temp;
				
				lowercaseString = _wcslwr( nameInfo->Name.Buffer );
				temp = wcsstr( lowercaseString, L"TargetDriver.sys");
				
				if( temp )
				{
					Data->IoStatus.Status = STATUS_NO_SUCH_FILE;
					Data->IoStatus.Information = 0;
					return FLT_PREOP_COMPLETE;
				}
			
				FltReleaseFileNameInformation( nameInfo );
			}
	}

    return FLT_PREOP_SUCCESS_NO_CALLBACK;
}

NTSTATUS DriverEntry( PDRIVER_OBJECT DriverObject,  PUNICODE_STRING RegistryPath)
{	
	NTSTATUS        status, ntStatus = STATUS_SUCCESS;
	UNICODE_STRING  ntUnicodeString;    
	UNICODE_STRING  ntWin32NameString, str;  
	PDEVICE_OBJECT  deviceObject;

	ULONG           i;

	RtlInitUnicodeString( &ntUnicodeString, NT_DEVICE_NAME );

	RtlInitUnicodeString( &ntWin32NameString, DOS_DEVICE_NAME );

	ntStatus = IoCreateSymbolicLink(&ntWin32NameString, &ntUnicodeString);

	if (!NT_SUCCESS(ntStatus))
	{
		return ntStatus;
	}
	
	ntStatus = IoCreateDevice(
		DriverObject,                  
		0,                            
		&ntUnicodeString,               
		FILE_DEVICE_UNKNOWN,          
		FILE_DEVICE_SECURE_OPEN,   
		FALSE,                         
		&deviceObject );

	if(!NT_SUCCESS(ntStatus))
	{

		IoDeleteSymbolicLink( &ntWin32NameString );
		return ntStatus;		
	}
    
	DriverObject->MajorFunction[IRP_MJ_CREATE] = CreateFunction;
	DriverObject->MajorFunction[IRP_MJ_CLOSE] = CloseFunction;
	DriverObject->MajorFunction[IRP_MJ_DEVICE_CONTROL] = DeviceControlFunction;
	DriverObject->DriverUnload = UnloadDriver;
	
	RtlInitUnicodeString( &str, L"\\Registry\\Machine\\System\\CurrentControlSet\\Services\\TargetDriver");
	ntStatus = ZwUnloadDriver( &str );

    ntStatus = FltRegisterFilter( DriverObject, &FilterRegistration, &FilterHandle );

    ASSERT( NT_SUCCESS( ntStatus ) );

    if (NT_SUCCESS( ntStatus )) 
	{

        ntStatus = FltStartFiltering( FilterHandle );

        if (!NT_SUCCESS( ntStatus )) {

            FltUnregisterFilter( FilterHandle );
        }
    }

	return ntStatus;
}

NTSTATUS CreateFunction(IN PDEVICE_OBJECT DeviceObject, IN PIRP pIrp)
{

	NTSTATUS   ntStatus;

	pIrp->IoStatus.Status = STATUS_SUCCESS;
	pIrp->IoStatus.Information = 0;

	IoCompleteRequest( pIrp, IO_NO_INCREMENT );
	
	return STATUS_SUCCESS;

}

NTSTATUS CloseFunction(IN PDEVICE_OBJECT DeviceObject, IN PIRP pIrp)
{
	NTSTATUS   ntStatus;
   
	pIrp->IoStatus.Status = STATUS_SUCCESS;
	pIrp->IoStatus.Information = 0;

	IoCompleteRequest( pIrp, IO_NO_INCREMENT );

	return STATUS_SUCCESS;

}

VOID UnloadDriver(IN PDRIVER_OBJECT DriverObject)
{
	KIRQL oldIrql;
	UNICODE_STRING uniWin32NameString;
	NTSTATUS  status;
	PDEVICE_OBJECT deviceObject = DriverObject->DeviceObject;
	

	RtlInitUnicodeString( &uniWin32NameString, DOS_DEVICE_NAME );

	IoDeleteSymbolicLink( &uniWin32NameString );

	if ( deviceObject != NULL )
	{
		IoDeleteDevice( deviceObject );
	}
}

NTSTATUS DeviceControlFunction(IN PDEVICE_OBJECT DeviceObject, IN PIRP pIrp)
{
	KIRQL  oldIrql; 
	
	
    PIO_STACK_LOCATION  IrpStack;
	ULONG  InputLength, OutputLength;

    NTSTATUS            ntStatus = STATUS_SUCCESS;

  
	pIrp->IoStatus.Status = ntStatus;

    IoCompleteRequest( pIrp, IO_NO_INCREMENT );

    return ntStatus;
}
